@@ -2,16 +2,15 @@ import socket
 import threading
 import logging
 import datetime
+import os
 
 
-
-
-from Server.utils import prepare_model,receive,save_model
+from Server.utils import prepare_model,receive,save_model,load_input
 logging.basicConfig(level=logging.NOTSET)
 
 
 class DatachannelServer:
-    def __init__(self, ip, port,clientsDB,server_model_path,model_type,listener_num = 100, gap_time=20):
+    def __init__(self, ip, port,clientsDB,server_model_path,model_type,golden_data_input,golden_data_output,score_fn,gap_time=20, listener_num = 100):
         self.ip = ip
         self.port = port
         self.server_model_path = server_model_path
@@ -20,6 +19,9 @@ class DatachannelServer:
         self.gap_time = gap_time
         self.clientsDB = clientsDB
         self.server = None
+        self.golden_data_input,self.golden_data_output = load_input(golden_data_input,golden_data_output)
+        self.score_fn = score_fn
+        self.received_values = {}
         logging.debug('Datachannel initialized')
     
         
@@ -49,6 +51,7 @@ class DatachannelServer:
         main_thread.start()
         logging.debug('main_thread started')
 
+    
     def calculate_average(self):
         if self.model_type == "pytorch":
             import torch
@@ -57,20 +60,26 @@ class DatachannelServer:
             if len(keys) == 0:
                 return None
             init_weights = self.received_values[keys[0]]
-        
+            averaging_weight = self.score_fn(init_weights(self.golden_data_input),self.golden_data_output)
+            sum_weighted_average = averaging_weight
+            with torch.no_grad():
+                for i in init_weights.parameters():
+                    i*=averaging_weight
             print("Before")
             for key in keys:
                 for i in self.received_values[key].parameters():
                     print(i)
                 print("Another Model")
             for mdl in range(1,len(keys)):
+                averaging_weight = self.score_fn(self.received_values[keys[mdl]](self.golden_data_input),self.golden_data_output)
+                sum_weighted_average += averaging_weight
                 with torch.no_grad():
                     for (i,j) in zip(init_weights.parameters(),self.received_values[keys[mdl]].parameters()):
-                        i+=j 
+                        i+=averaging_weight*j 
             
             with torch.no_grad():
                 for i in init_weights.parameters():
-                    i/=len(keys)
+                    i/=sum_weighted_average
             
             print("After")
             for i in init_weights.parameters():
@@ -88,19 +97,28 @@ class DatachannelServer:
                 for layer in model.layers:
                     print(layer.get_weights())
                     print("\n")
-
+            model = models[0]
+            averaging_weight = self.score_fn(model(self.golden_data_input),self.golden_data_output)
+            sum_weighted_average = averaging_weight
+            for layer in model.layers:
+                curr_weights = layer.get_weights()
+                for i in range(len(curr_weights)):
+                    curr_weights[i] = curr_weights[i] * averaging_weight
+                layer.set_weights(curr_weights)
             for model in models[1:]:
+                averaging_weight = self.score_fn(model(self.golden_data_input),self.golden_data_output)
+                sum_weighted_average += averaging_weight
                 for (layer0,layer1) in zip(models[0].layers,model.layers):
                     curr_weights = layer0.get_weights()
                     weights = layer1.get_weights()
                     for i in range(len(curr_weights)):
-                        curr_weights[i] = curr_weights[i] + weights[i]
+                        curr_weights[i] = curr_weights[i] + averaging_weight*weights[i]
                     layer0.set_weights(curr_weights)
         
             for layer in models[0].layers:
                 curr_weights = layer.get_weights()
                 for i in range(len(curr_weights)):
-                    curr_weights[i] = curr_weights[i] / len(models)
+                    curr_weights[i] = curr_weights[i] / sum_weighted_average
                 layer.set_weights(curr_weights)
             print("After")
             for layer in models[0].layers:
@@ -123,11 +141,17 @@ class DatachannelServer:
             pass
         averaged_model = self.calculate_average()
         self.server_controller.increase_version()
-        whole_path = self.server_model_path + '/' + 'model_' + str(self.server_controller.get_version()) + '.pt'
+
+        whole_path = self.server_model_path + '/' + 'model_' + str(self.server_controller.get_version())
+        if self.model_type == "pytorch":
+            whole_path += '.pt'
+        elif self.model_type == "tensorflow":
+            whole_path += '.h5'
         print(whole_path)
+        for f in os.listdir(self.server_model_path):
+            os.remove(os.path.join(self.server_model_path, f))
         save_model(self.model_type,whole_path,averaged_model)
         # Interrupt server.accept() with fake connection
-        
         fake_server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         IP_address = self.ip
         Port = self.port
